require 'drb'

=begin
== Center
Derive your own subclass and launch() it. Launch() will wait until the thread
of the Center ends.
=end
class Center
  Port = 9948
  LocalServer = "localhost:#{Port}"

  def initialize(name, host="127.0.0.1", port=Port)
    @allow_uri = "druby://#{host}:#{port}"
    @server = DRb.start_service(@allow_uri, self)
    at_exit {  # be nice (catches ^C and more)
      p "Taking #{name} down!"
      @server.thread.kill
    }
  end
  attr_reader :server

  # Test-whether-center-is-running
  def ping() "pong" end

  # Ruby iPAQ
  def menu() Thread.new { load 'ipaq/menu.rb' } end
  def cpu_load() Thread.new { load 'ipaq/cpu.rb' } end
  def battery() Thread.new { load 'ipaq/power.rb' } end
  def clock() Thread.new { load 'ipaq/time.rb' } end
  def light() Thread.new { load 'ipaq/light.rb' } end
  def cpu_freq() Thread.new { load 'ipaq/cpu-freq.rb' } end

  # Ruby PIM
  def address() Thread.new { load 'pim/address/wise.rb' } end
  # Have to skip agenda until it is Wise-compatible
  def todo() Thread.new { load 'pim/todo/wise.rb' } end

  begin
    DRb.start_service()
    d = DRbObject.new(nil, "druby://localhost:#{Center::Port}")
    d.ping
  rescue
    puts "Starting Ruby/Center..."  # if $DEBUG
    fork {
      center = Center.new("Ruby/Center")
      center.server.thread.join
    }
  end
end
