=begin
Simple XML parser, which constructs a tree of XmlElts
Modeled after XMLtp, by Thomas Weidenfeller, http://mitglied.tripod.de/xmltp/

Parses syntactically only; does not look for "&xxx;" constructions; gives
really bad errors when parsing fails, because the xml is supposed to be
generated by applications, i.e. contain no errors.

Meant partially as an exercise to become familiar with Ruby (I started looking
at Ruby 2 V 2001; looks like a nice language, but of course I will only learn
the disadvantages when I am a beyond-average user); meant partially to unlock
my data for access in Ruby.

:Author: Kero van Gelder
:Copyright: Kero van Gelder, the GNU General Public License applies
:First version: 2 V 2001
:Last Revision: 3 V 2001
=end

require 'kero/report'
require 'kero/xmlelt'

=begin
The XML Parser itself. One instance can be used to parse several I/O-streams.
=end
class XmlParser
  attr :curChar

  def initialize(elements, anon=AnonXml)
    @hash = Hash.new()
    if elements != nil then
      elements.each do |cls|
	@hash.store(cls.xmlName(), cls)
      end
    end
    @anon = anon
  end

  def nextChar()
    @curChar = @f.read(1)
  end

  def parseSpace()
    while /\s/ =~ @curChar
      nextChar()
    end
  end

  def parseChar(needChar)
    parseSpace()
    if @curChar == needChar
      nextChar()
    else
      raise "Expected '"+needChar.to_s+"', found '"+@curChar.to_s()+"'"
    end
  end

  def parseIdent()
    parseSpace()
    ident = ""
    while /[a-zA-Z0-9_]/ =~ @curChar
      ident += @curChar
      nextChar()
    end
    #p "ident: "+ident
    ident
  end

  def parseString()
    parseChar('"')
    string = ""
    while (@curChar != nil && @curChar != '"')
      string += @curChar
      nextChar()
    end
    parseChar('"')
    #p "string: "+string end
    string
  end

  def parseAttributes(elt)
    attrIdent = parseIdent()
    while (attrIdent != "")
      begin
	parseChar('=')
	attrString = parseString()
	attrString.gsub!("[\\s]+", " ")
	attrString.strip!()
	elt.addAttribute(attrIdent, attrString)
	debug " "+attrIdent+"=\""+attrString+"\"", false
      rescue
	# continue, please
	elt.addAttribute(attrIdent)
	debug " "+attrIdent, false
      end
      attrIdent = parseIdent()
    end
  end

  def parseContent(elt)
    content = ""
    getMore = true
    while (@curChar != nil && getMore)
      if @curChar == '<'
	nextChar()
	if (@curChar == '/')
	  # closing tag
	  getMore = false
	  nextChar()
	else
	  # opening tag, recurse
	  child = parseAfterLT()
	  elt.addChild(child)
	  #elt.addContent(content) ???
	  #content = "" ???
	end
      else
	content += @curChar
	nextChar()
      end
    end
    content.gsub!("[\\s]+", " ")
    content.strip!()
    if (content != "") then
      d content
      elt.addContent(content)
    end
    content
  end

  def parseAfterLT()
    tag = parseIdent()
    debug "<"+tag, false
    elt = nil
    if @hash.has_key? tag
      cls = @hash[tag]
      elt = cls.new()
    else
      elt = @anon.new(tag)
    end
    parseAttributes(elt)
    parseSpace()
    if @curChar == '/'
      # Also end, no content
      debug "/"
      nextChar()
      parseChar('>')
    elsif @curChar == '>'
      # Get the content
      debug ">"
      nextChar()
      parseContent(elt) # recognizes EndOfContent by scanning past '</'
      tag2 = parseIdent()
      if (tag2 != tag)
        raise "closing tag "+tag2+" does not match opening tag "+tag
      end
      parseChar('>')
    else
      raise "Expected '/>' or '>', found #{@curChar}"
    end
    return elt
  end

  def parseStream(io)
    @f = io
    result = []
    nextChar()
#    begin
      parseSpace()
      while (@curChar != nil) do
	parseChar('<')
	result << parseAfterLT()
	parseSpace()
      end
#    rescue
#      # EOF reached; check whether we are nested level zero?
#    end
    #debug "Retrieved list: "+result.to_s
    result
  end
end
