=begin
Each possible object in the resulting XML tree must be of a class that includes
this module; only then can the XmlParser build the tree.

Note that you have to create a class method xmlName() as well (but a Ruby
module can not enforce this; neither can a Java interface, btw). If you forget
this, XmlParser.new() will generate a RunTimeError.

For convenience, a default printing routine is given in this module. It is
/strongly/ adviced you overload childrenStr() in any XmlElt that you use that
can have children
=end
module XmlElt
  attr_reader :attrib, :children
  def addAttribute(attrib, value=nil)
    # nil means attribute without value, string means with value
    @attrib.store(attrib, value)  # if value != ""
    # In a similar way, when set (elsewhere), REMOVE when empty string !!!
  end
  #def addContent(content)
  #  @content << content
  #end
  # when adding the child, you can set the parent of the child, if you wish
  def addChild(child)
    @children << child
  end
  def removeChild(child)
    @children.delete(child)  # slowish...
  end

  def attribStr()
    result = ""
    @attrib.each_pair do |attrib, value|
      result += " "+attrib
      result += '="'+value+'"'  if value
    end  if @attrib
    result
  end

  def openingTagStr(indent_str)
    "#{indent_str}<#{type().xmlName()}#{attribStr()}>"
  end

  #def contentStr(indent_str)
  #  result = ""
  #  @content.each do |content|
  #    result += content.to_s("\t#{indent_str}")
  #  end  if @content
  #  result
  #end

  def childrenStr(indent_str)
    result = ""
    deeper = "\t#{indent_str}"
    @children.each { |entry|
      result += entry.to_s(deeper) + "\n"
    }  if @children
    result
  end

  def closingTagStr(indent_str)
    "#{indent_str}</#{type().xmlName()}>"
  end

  def to_s(indent_str="")
    result = openingTagStr(indent_str)
    # content = contentStr(indent_str)
    content = childrenStr(indent_str)
    if (content.strip() != "")
      result += "\n"+content+closingTagStr(indent_str)+"\n"
    else
      result[-1, 0] = "/"
    end
    result
  end
end

class AnonXml
  include XmlElt
  def initialize(name)
    @name = name
    @attrib = {}
    @children = []
    #@content = []
  end
  attr_reader :name

  def openingTagStr(indent_str="")
    "#{indent_str}<#{@name}#{attribStr}>"
  end
  def closingTagStr(level=0)
    "#{indent_str}</#{@name}>"
  end
end
